#version 330 core

uniform sampler2D colorTexture;
uniform sampler2D shadowMap;
uniform vec3 camPos;

in vec3 theNormal;
in vec2 fragmentUV;
in vec3 fragmentPos; // position of fragment in world space
in vec4 smCoords;

layout (location = 0) out vec4 outputColor;
layout (location = 1) out vec4 outputMotion;

const float k_d = 0.25; // diffuse coefficient
const float p = 5; // specular parameter
const float k_s = 0.5; // specular reflection coefficient

const float texSize = 5; // the texture is stretched over texSize units
const float texSizeMul = 1 / (2 * texSize);

void main()
{
	vec3 l = normalize(vec3(1.5f, 3.5f, -4.f)); // directional light vector
	vec3 c = texture(colorTexture, vec2((fragmentPos.x + fragmentPos.y) * texSizeMul, (fragmentPos.z + fragmentPos.y) * texSizeMul)).rgb;
	vec3 v = normalize(camPos - fragmentPos);
	vec3 h = normalize(l + v);
	float lightI = clamp(dot(l, theNormal), 0, 1);

	float smShadowing;
	if (abs(smCoords.x) >= 1.0 ||
		abs(smCoords.y) >= 1.0 ||
		abs(smCoords.z) >= 1.0 ||
		smCoords.w <= 0.0) {
		smShadowing = 1.0; // outside bounds or shadow
	} else {
		vec4 smCoordsW = smCoords / smCoords.w * .5 + .5; // norm. and move to pos. range
		
		vec2 moments = texture2D(shadowMap, smCoordsW.xy).rg; // get moments saved in SM
	
		if (moments.r < smCoordsW.z) {

			// Chebyshev method for determining whether full shadow or border
			
			// variance
			float v = max(moments.g - moments.r * moments.r, .000002); // add zeros if necessary
			
			float d = smCoordsW.z - moments.r;

			smShadowing = v / (v + d * d);
		}
		else
			smShadowing = 1.0; // in front of thing throwing the shadow
	}

	outputColor = vec4(smShadowing * (k_d * lightI + 0.65 * c + k_s * lightI * pow(clamp(dot(theNormal, h), 0, 1), p)), 1);
	outputMotion = vec4(0);
}
