#version 330 core

in vec2 fragmentPos;

out vec4 outputColor;

uniform sampler2D image0; // screen
uniform sampler2D image1; // current depth buffer
uniform sampler2D image2; // model velocity texture
uniform mat4 vp;
uniform mat4 prevVp;
uniform mat4 invVp;
uniform float deltaTime;

const int numSamples = 20;
const float mvf = 0.00002; // model velocity factor
const float vf = 0.0002; // velocity factor

void main()
{
	vec3 pxColor = texture(image0, fragmentPos).rgb;
	float currentDepth = texture(image1, fragmentPos).r;
	vec2 mVelocity = texture(image2, fragmentPos).rg;
	vec2 modelVelocity = vec2(0);
	// model motion blur only for the parts where a model has moved individually
	if (mVelocity != vec2(0)) {
		modelVelocity = ((mVelocity - 0.5) * 4 - 1);
		//modelVelocity *= -1;
	}

	// viewport position of current fragment in [-1,1]
	vec4 H = vec4(fragmentPos.x * 2 - 1, (1 - fragmentPos.y) * 2 - 1, currentDepth, 1);
	// world position of current fragment
	vec4 D = invVp * H;
	vec4 worldPos = D / D.w;

	vec4 currentPos = H;
	// previous world position of current fragment
	vec4 previousPos = prevVp * worldPos;
	previousPos /= previousPos.w;

	vec2 velocity = modelVelocity * mvf / deltaTime + (currentPos - previousPos).xy * vf / deltaTime;
	vec3 color = pxColor;

	if (abs(velocity.x) + abs(velocity.y) > 0) {
		vec2 texPos = fragmentPos;

		// sampling along velocity direction
		for (int i = 1; i < numSamples; ++i) {
			texPos += velocity;
			color += texture(image0, clamp(texPos, 0, 1)).rgb;
		}
		color /= numSamples;
		outputColor = vec4(color, 1);
	} else {
		outputColor = vec4(pxColor, 1);
	}
}
